"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MultiplePackagesBuilder = void 0;
var _path = _interopRequireDefault(require("path"));
var _listr = _interopRequireDefault(require("listr"));
var _BasePackagesBuilder = require("./BasePackagesBuilder");
var _chalk = require("chalk");
var _utils = require("../../utils");
var _child_process = require("child_process");
var _serializeError = require("serialize-error");
const WORKER_PATH = _path.default.resolve(__dirname, "worker.js");
class MultiplePackagesBuilder extends _BasePackagesBuilder.BasePackagesBuilder {
  async build() {
    const packages = this.packages;
    const context = this.context;
    const inputs = this.inputs;
    const getBuildDuration = (0, _utils.measureDuration)();
    context.info(`Building %s packages...`, packages.length);
    const tasksList = packages.map(pkg => {
      return {
        title: this.getPackageLabel(pkg),
        task: () => {
          return new Promise((resolve, reject) => {
            const buildConfig = JSON.stringify({
              ...inputs,
              package: {
                paths: pkg.paths
              }
            });
            const child = (0, _child_process.fork)(WORKER_PATH, [buildConfig], {
              silent: true
            });

            // We only send one message from the child process, and that is the error, if any.
            child.on("message", serializedError => {
              const error = (0, _serializeError.deserializeError)(serializedError);
              reject(new Error("Build failed.", {
                cause: {
                  pkg,
                  error
                }
              }));
            });

            // Handle child process error events
            child.on("error", error => {
              reject(new Error("Build failed.", {
                cause: {
                  pkg,
                  error
                }
              }));
            });

            // Handle child process exit and check for errors
            child.on("exit", code => {
              if (code !== 0) {
                const error = new Error(`Build process exited with code ${code}.`);
                reject(new Error("Build failed.", {
                  cause: {
                    pkg,
                    error
                  }
                }));
                return;
              }
              resolve();
            });
          });
        }
      };
    });
    const tasks = new _listr.default(tasksList, {
      concurrent: true,
      exitOnError: false
    });
    await tasks.run().catch(err => {
      console.log();
      context.error(`Failed to build all packages. For more details, check the logs below.`);
      console.log();

      /**
       * Seems List package has wrong types or this never worked.
       */
      err.errors.forEach((err, i) => {
        const {
          pkg,
          error
        } = err.cause;
        const number = `${i + 1}.`;
        const name = context.error.hl(pkg.name);
        const relativePath = (0, _chalk.gray)(`(${pkg.paths.relative})`);
        const title = [number, name, relativePath].join(" ");
        console.log(title);
        console.log(error.message);
        if (inputs.debug) {
          console.log(error.stack);
        }
      });
      throw new Error(`Failed to build all packages.`);
    });
    console.log();
    context.success(`Built ${packages.length} packages in ${getBuildDuration()}.`);
  }
  getPackageLabel(pkg) {
    const pkgName = pkg.name;
    const pkgRelativePath = (0, _chalk.gray)(`(${pkg.paths.relative})`);
    return `${pkgName} ${pkgRelativePath}`;
  }
}
exports.MultiplePackagesBuilder = MultiplePackagesBuilder;

//# sourceMappingURL=MultiplePackagesBuilder.js.map