"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.newWatch = void 0;
var _chalk = _interopRequireDefault(require("chalk"));
var _path = _interopRequireDefault(require("path"));
var _utils = require("@webiny/cli/utils");
var _get = _interopRequireDefault(require("lodash/get"));
var _merge = _interopRequireDefault(require("lodash/merge"));
var _utils2 = require("../utils");
var _getIotEndpoint = require("./newWatch/getIotEndpoint");
var _listLambdaFunctions = require("./newWatch/listLambdaFunctions");
var _listPackages = require("./newWatch/listPackages");
var _PackagesWatcher = require("./newWatch/watchers/PackagesWatcher");
var _initInvocationForwarding = require("./newWatch/initInvocationForwarding");
var _replaceLambdaFunctions = require("./newWatch/replaceLambdaFunctions");
// Do not allow watching "prod" and "production" environments. On the Pulumi CLI side, the command
// is still in preview mode, so it's definitely not wise to use it on production environments.
const WATCH_DISABLED_ENVIRONMENTS = ["prod", "production"];
const newWatch = async (inputs, context) => {
  // 1. Initial checks for deploy and build commands.
  if (!inputs.folder && !inputs.package) {
    throw new Error(`Either "folder" or "package" arguments must be passed. Cannot have both undefined.`);
  }
  const project = (0, _utils.getProject)();
  const projectApplicationSpecified = !!inputs.folder;

  // Detect if an app alias was provided.
  let projectApplication = undefined;
  if (projectApplicationSpecified) {
    if (project.config.appAliases) {
      const appAliases = project.config.appAliases;
      if (appAliases[inputs.folder]) {
        inputs.folder = appAliases[inputs.folder];
      }
    }

    // Get project application metadata. Will throw an error if invalid folder specified.
    projectApplication = (0, _utils.getProjectApplication)({
      cwd: _path.default.join(process.cwd(), inputs.folder)
    });

    // If exists - read default inputs from "webiny.application.ts" file.
    inputs = (0, _merge.default)({}, (0, _get.default)(projectApplication, "config.cli.watch"), inputs);

    /**
     * We can safely assume that the project application is defined here.
     *
     * Check if there are any plugins that need to be registered.
     */
    if (projectApplication.config.plugins) {
      context.plugins.register(projectApplication.config.plugins);
    }

    // Load env vars specified via .env files located in project application folder.
    await (0, _utils2.loadEnvVariables)(inputs, context);
  }
  if (projectApplicationSpecified && !inputs.env) {
    throw new Error(`Please specify environment, for example "dev".`);
  }
  if (WATCH_DISABLED_ENVIRONMENTS.includes(inputs.env)) {
    if (!inputs.allowProduction) {
      throw new Error(`${_chalk.default.red("webiny watch")} command cannot be used with production environments.`);
    }
  }
  const hookArgs = {
    context,
    env: inputs.env,
    variant: inputs.variant,
    inputs,
    projectApplication
  };
  await (0, _utils2.runHook)({
    hook: "hook-before-watch",
    args: hookArgs,
    context
  });
  console.log();
  const packages = await (0, _listPackages.listPackages)({
    inputs
  });
  const packagesWatcher = new _PackagesWatcher.PackagesWatcher({
    packages,
    context,
    inputs
  });
  if (!projectApplicationSpecified || !projectApplication) {
    await packagesWatcher.watch();
    return;
  }

  // Maximum of 15minutes in seconds can be passed.
  if (inputs.increaseTimeout && inputs.increaseTimeout > 900) {
    throw new Error(`When increasing the timeout, the maximum value that can be passed is 900 seconds (15 minutes).`);
  }
  const functionsList = (0, _listLambdaFunctions.listLambdaFunctions)({
    env: inputs.env,
    folder: inputs.folder,
    variant: inputs.variant,
    whitelist: inputs.function
  });
  const deployCommand = `yarn webiny deploy ${projectApplication.id} --env ${inputs.env}`;
  const learnMoreLink = "https://webiny.link/local-aws-lambda-development";
  const troubleshootingLink = learnMoreLink + "#troubleshooting";
  if (functionsList.meta.count === 0) {
    // If functions exist, but none are selected for watching, show a warning.
    if (functionsList.meta.totalCount > 0) {
      context.info(["No AWS Lambda functions will be invoked locally. If this is unexpected, you can try the following:", " ‣ stop the current development session", " ‣ redeploy the %s application by running %s command", " ‣ start a new %s session by rerunning %s command", "", "Learn more: %s"].join("\n"), projectApplication.name, deployCommand, "webiny watch", "webiny watch", troubleshootingLink);
      console.log();
    }
    await packagesWatcher.watch();
    return;
  }
  context.info(`Local AWS Lambda development session started.`);
  context.warning(`Note that once the session is terminated, the %s application will no longer work. To fix this, you %s redeploy it via the %s command. Learn more: %s.`, projectApplication.name, "MUST", deployCommand, learnMoreLink);
  context.debug("The events for following AWS Lambda functions will be forwarded locally: ", functionsList.list.map(fn => fn.name));
  console.log();
  const {
    default: exitHook
  } = await import(/* webpackChunkName: "exit-hook" */"exit-hook");
  exitHook(() => {
    console.log();
    console.log();
    context.info(`Terminating local AWS Lambda development session.`);
    context.warning(`Note that once the session is terminated, the %s application will no longer work. To fix this, you %s redeploy it via the %s command. Learn more: %s.`, projectApplication?.name, "MUST", deployCommand, learnMoreLink);
  });
  const deploymentId = (0, _utils2.getDeploymentId)({
    env: inputs.env,
    variant: inputs.variant
  });
  const iotEndpointTopic = `webiny-watch-${deploymentId}`;
  const iotEndpoint = await (0, _getIotEndpoint.getIotEndpoint)({
    env: inputs.env,
    variant: inputs.variant
  });
  const sessionId = new Date().getTime();
  const increaseTimeout = inputs.increaseTimeout;

  // We want to ensure a Pulumi refresh is made before the next deploy.
  (0, _utils2.setMustRefreshBeforeDeploy)(context);

  // Ignore promise, we don't need to wait for this to finish.
  (0, _replaceLambdaFunctions.replaceLambdaFunctions)({
    context,
    env: inputs.env,
    folder: inputs.folder,
    variant: inputs.variant,
    iotEndpoint,
    iotEndpointTopic,
    sessionId,
    functionsList,
    increaseTimeout
  });
  let inspector = undefined;
  if (inputs.inspect) {
    inspector = require("inspector");
    inspector.open(9229, "127.0.0.1");
    console.log();
    exitHook(() => {
      inspector.close();
    });
  }

  // Ignore promise, we don't need to wait for this to finish.
  (0, _initInvocationForwarding.initInvocationForwarding)({
    iotEndpoint,
    iotEndpointTopic,
    functionsList,
    sessionId
  });
  await packagesWatcher.watch();
};
exports.newWatch = newWatch;

//# sourceMappingURL=newWatch.js.map