"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initInvocationForwarding = void 0;
var _path = _interopRequireDefault(require("path"));
var _worker_threads = require("worker_threads");
var _gzip = require("@webiny/utils/compression/gzip");
var _mqtt = _interopRequireDefault(require("mqtt"));
const WEBINY_WATCH_FN_INVOCATION_EVENT = "webiny.watch.functionInvocation";
const WEBINY_WATCH_FN_INVOCATION_RESULT_EVENT = "webiny.watch.functionInvocationResult";
const WATCH_WORKER_PATH = _path.default.join(__dirname, "localInvocationWorker.js");
const jsonStringifyAndCompress = input => {
  const jsonStringInput = JSON.stringify(input);
  return (0, _gzip.compress)(jsonStringInput);
};
const decompressAndJsonParse = async input => {
  const inputBuffer = Buffer.from(input);
  const jsonStringResult = await (0, _gzip.decompress)(inputBuffer);
  const value = jsonStringResult?.toString ? jsonStringResult.toString("utf-8") : jsonStringResult;
  return JSON.parse(value);
};
const initInvocationForwarding = async ({
  iotEndpoint,
  iotEndpointTopic,
  sessionId,
  functionsList
}) => {
  const client = await _mqtt.default.connectAsync(iotEndpoint);
  await client.subscribeAsync(iotEndpointTopic);
  client.on("message", async (_, message) => {
    const payload = JSON.parse(message.toString());
    if (payload.eventType !== WEBINY_WATCH_FN_INVOCATION_EVENT) {
      return;
    }
    if (payload.data.sessionId !== sessionId) {
      return;
    }
    const invokedLambdaFunction = functionsList.list.find(lambdaFunction => lambdaFunction.name === payload.data.functionName);
    if (!invokedLambdaFunction) {
      throw new Error(`Lambda function "${payload.data.functionName}" not found.`);
    }
    try {
      const result = await new Promise(async (resolve, reject) => {
        const worker = new _worker_threads.Worker(WATCH_WORKER_PATH, {
          env: {
            ...payload.data.env,
            WEBINY_WATCH_LOCAL_INVOCATION: "1"
          },
          workerData: {
            handler: {
              path: invokedLambdaFunction.path,
              args: await decompressAndJsonParse(payload.data.compressedArgs)
            }
          }
        });
        const {
          default: exitHook
        } = await import(/* webpackChunkName: "exit-hook" */"exit-hook");
        const unsubscribeExitHook = exitHook(async () => {
          await worker.terminate();
        });
        worker.on("message", message => {
          unsubscribeExitHook();
          const {
            success,
            result,
            error
          } = JSON.parse(message);
          if (success) {
            resolve(result);
          } else {
            reject(error);
          }
        });
      });
      await client.publish(iotEndpointTopic, JSON.stringify({
        eventType: WEBINY_WATCH_FN_INVOCATION_RESULT_EVENT,
        eventId: new Date().getTime(),
        data: {
          originalEventId: payload.eventId,
          compressedResult: await jsonStringifyAndCompress(result),
          compressedError: null
        }
      }));
    } catch (error) {
      console.log(error);
      await client.publish(iotEndpointTopic, JSON.stringify({
        eventType: WEBINY_WATCH_FN_INVOCATION_RESULT_EVENT,
        eventId: new Date().getTime(),
        data: {
          originalEventId: payload.eventId,
          compressedResult: null,
          compressedError: await jsonStringifyAndCompress({
            message: error.message,
            stack: error.stack
          })
        }
      }));
    }
  });
  return client;
};
exports.initInvocationForwarding = initInvocationForwarding;

//# sourceMappingURL=initInvocationForwarding.js.map