"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MultiplePackagesWatcher = void 0;
var _path = _interopRequireDefault(require("path"));
var _chalk = _interopRequireDefault(require("chalk"));
var _BasePackagesWatcher = require("./BasePackagesWatcher");
var _utils = require("../../../utils");
var _child_process = require("child_process");
var _serializeError = require("serialize-error");
const WORKER_PATH = _path.default.resolve(__dirname, "worker.js");
class MultiplePackagesWatcher extends _BasePackagesWatcher.BasePackagesWatcher {
  async watch() {
    const packages = this.packages;
    const context = this.context;
    const inputs = this.inputs;
    context.info(`Watching %s packages...`, packages.length);
    if (inputs.debug) {
      context.debug("The following packages will be watched for changes:");
      packages.forEach(item => console.log("‣ " + item.name));
    }
    const tasksList = [];
    for (let i = 0; i < packages.length; i++) {
      const pkg = packages[i];
      const log = createLog({
        context,
        packageName: pkg.name
      });
      tasksList.push(new Promise((resolve, reject) => {
        const buildConfig = JSON.stringify({
          ...inputs,
          package: {
            paths: pkg.paths
          }
        });
        const child = (0, _child_process.fork)(WORKER_PATH, [buildConfig], {
          env: {
            ...process.env
          },
          stdio: ["pipe", "pipe", "pipe", "ipc"] // Use "pipe" to handle custom output
        });

        // Prefix each line of child stdout with the package name
        if (child.stdout) {
          child.stdout.on("data", chunk => {
            log(chunk.toString());
          });
        }

        // Prefix each line of child stderr with the package name
        if (child.stderr) {
          child.stderr.on("data", chunk => {
            log(chunk.toString(), "error");
          });
        }

        // We only send one message from the child process, and that is the error, if any.
        child.on("message", serializedError => {
          const error = (0, _serializeError.deserializeError)(serializedError);
          reject(new Error("Build failed.", {
            cause: {
              pkg,
              error
            }
          }));
        });

        // Handle child process error events
        child.on("error", error => {
          reject(new Error("Build failed.", {
            cause: {
              pkg,
              error
            }
          }));
        });

        // Handle child process exit and check for errors
        child.on("exit", code => {
          if (code !== 0) {
            const error = new Error(`Build process exited with code ${code}.`);
            reject(new Error("Build failed.", {
              cause: {
                pkg,
                error
              }
            }));
            return;
          }
          resolve();
        });
      }));
    }
    await Promise.all(tasksList);
  }
}
exports.MultiplePackagesWatcher = MultiplePackagesWatcher;
const createLog = ({
  context,
  packageName
}) => {
  const prefix = _chalk.default.hex((0, _utils.getRandomColorForString)(packageName))(packageName) + ": ";
  return (message, type) => {
    let send = prefix + message;
    if (type) {
      if (type === "error") {
        send = context.error.hl(send);
      }
      if (type === "warn") {
        send = context.warning.hl(send);
      }
    }
    console.log(send);
  };
};
module.exports = {
  MultiplePackagesWatcher
};

//# sourceMappingURL=MultiplePackagesWatcher.js.map