"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDeployedSystems = void 0;
var _stacks = require("../../utils/stacks");
var _utils = require("../../utils");
var _DeployedSystem = require("./DeployedSystem");
var _ora = _interopRequireDefault(require("ora"));
const validateVariantNames = params => {
  const {
    primary,
    secondary
  } = params;

  /**
   * We cannot check primary/secondary variables with ! because one of them might be empty - systems deployed without a variant.
   * Both cannot be empty.
   *
   * Also, primary and secondary variants cannot be the same.
   */
  if (!primary && !secondary) {
    throw new _utils.GracefulError("Primary and secondary variants cannot be empty.");
  } else if (primary === secondary) {
    throw new _utils.GracefulError("Primary and secondary variants cannot be the same.");
  }
};
const getDeployedSystems = async params => {
  const {
    env,
    context,
    folders
  } = params;
  validateVariantNames(params);
  const message = `Fetching primary and secondary stacks in "${env}" environment...`;
  const spinner = (0, _ora.default)(message);
  spinner.start();
  const stacks = await (0, _stacks.getApplicationsStacksOutput)({
    cwd: context.project.root,
    env,
    variants: [params.primary, params.secondary],
    context,
    folders
  });
  const primaryStacks = stacks.filter(stack => stack.variant === params.primary);
  if (primaryStacks.length === 0) {
    spinner.fail(`${message} failed. Could not find primary variant${params.primary ? ` "${params.primary}"` : ""}.`);
    throw new _utils.GracefulError(`Primary variant${params.primary ? ` "${params.primary}"` : ""} not found.`);
  }
  const secondaryStacks = stacks.filter(stack => stack.variant === params.secondary);
  if (secondaryStacks.length === 0) {
    spinner.fail(`${message} failed. Could not find secondary variant${params.secondary ? ` "${params.secondary}"` : ""}.`);
    throw new _utils.GracefulError(`Secondary variant${params.secondary ? ` "${params.secondary}"` : ""} not found.`);
  }
  spinner.succeed(`${message} done.`);
  const createDeployedSystem = (0, _DeployedSystem.createDeployedSystemFactory)({
    folders
  });
  try {
    return {
      primary: createDeployedSystem({
        stacks: primaryStacks
      }),
      secondary: createDeployedSystem({
        stacks: secondaryStacks
      })
    };
  } catch (ex) {
    spinner.fail(ex.message);
    throw ex;
  }
};
exports.getDeployedSystems = getDeployedSystems;

//# sourceMappingURL=getDeployedSystems.js.map