"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.watchCommand = void 0;
var _os = _interopRequireDefault(require("os"));
var _fs = _interopRequireDefault(require("fs"));
var _chalk = _interopRequireDefault(require("chalk"));
var _path = _interopRequireDefault(require("path"));
var _utils = require("@webiny/cli/utils");
var _get = _interopRequireDefault(require("lodash/get"));
var _merge = _interopRequireDefault(require("lodash/merge"));
var _simpleOutput = _interopRequireDefault(require("./watch/output/simpleOutput"));
var _listPackages = require("./watch/listPackages");
var _fastGlob = _interopRequireDefault(require("fast-glob"));
var _watchPackages = require("./watch/watchPackages");
var _PackagesWatcher = require("./watch/watchers/PackagesWatcher");
var _utils2 = require("../utils");
var _env = require("../utils/env");
// Do not allow watching "prod" and "production" environments. On the Pulumi CLI side, the command
// is still in preview mode, so it's definitely not wise to use it on production environments.
const WATCH_DISABLED_ENVIRONMENTS = ["prod", "production"];
const PULUMI_WATCH_SUPPORTED = _os.default.platform() !== "win32";

// Note: we are not using `createPulumiCommand` here because this command has a bit specific
// behaviour which is not encapsulated by `createPulumiCommand`. Maybe we can improve in the future.
const watchCommand = async (inputs, context) => {
  context.info(`With the %s release, we've introduced %s — a major update to the %s command and the overall backend development experience. This feature is currently in beta, but if you're interested in trying it out or learning more, check out the docs: https://webiny.link/local-aws-lambda-development`, "5.41.0", "local AWS Lambda development", "webiny watch");
  console.log();

  // 1. Initial checks for deploy and build commands.
  if (!inputs.folder && !inputs.package) {
    throw new Error(`Either "folder" or "package" arguments must be passed. Cannot have both undefined.`);
  }
  let projectApplication;
  if (inputs.folder) {
    // Detect if an app alias was provided.
    const project = (0, _utils.getProject)();
    if (project.config.appAliases) {
      const appAliases = project.config.appAliases;
      if (appAliases[inputs.folder]) {
        inputs.folder = appAliases[inputs.folder];
      }
    }

    // Get project application metadata. Will throw an error if invalid folder specified.
    projectApplication = (0, _utils.getProjectApplication)({
      cwd: _path.default.join(process.cwd(), inputs.folder)
    });

    // If exists - read default inputs from "webiny.application.ts" file.
    inputs = (0, _merge.default)({}, (0, _get.default)(projectApplication, "config.cli.watch"), inputs);

    // Check if there are any plugins that need to be registered.
    if (projectApplication.config.plugins) {
      context.plugins.register(projectApplication.config.plugins);
    }

    // Load env vars specified via .env files located in project application folder.
    await (0, _utils2.loadEnvVariables)(inputs, context);
  }
  inputs.build = inputs.build !== false;
  inputs.deploy = projectApplication && inputs.deploy !== false;
  if (inputs.deploy && !inputs.env) {
    throw new Error(`Please specify environment, for example "dev".`);
  }
  if (WATCH_DISABLED_ENVIRONMENTS.includes(inputs.env)) {
    if (!inputs.allowProduction) {
      throw new Error(`${_chalk.default.red("webiny watch")} command cannot be used with production environments.`);
    }
  }
  if (!inputs.build && !inputs.deploy) {
    throw new Error(`Both re-build and re-deploy actions were disabled, can't continue.`);
  }
  const hookArgs = {
    context,
    env: inputs.env,
    variant: inputs.variant,
    inputs,
    projectApplication
  };
  await (0, _utils2.runHook)({
    hook: "hook-before-watch",
    args: hookArgs,
    context
  });
  console.log();

  // TODO: separate the rest of the code below into separate "watcher" classes.
  // TODO: This was done just because of the time constraints.
  if (!inputs.deploy) {
    const packages = await (0, _listPackages.listPackages)({
      inputs
    });
    const packagesWatcher = new _PackagesWatcher.PackagesWatcher({
      packages,
      context,
      inputs
    });
    await packagesWatcher.watch();
    return;
  }

  // 1.1. Check if the project application and Pulumi stack exist.
  const PULUMI_SECRETS_PROVIDER = process.env.PULUMI_SECRETS_PROVIDER;
  if (inputs.deploy && projectApplication) {
    const {
      env,
      variant
    } = inputs;
    await (0, _utils2.login)(projectApplication);
    const pulumi = await (0, _utils2.getPulumi)({
      projectApplication
    });
    const stackName = (0, _utils2.getStackName)({
      env,
      variant
    });
    let stackExists = true;
    try {
      await pulumi.run({
        command: ["stack", "select", stackName],
        args: {
          secretsProvider: PULUMI_SECRETS_PROVIDER
        },
        execa: {
          env: (0, _env.createEnvConfiguration)({
            configurations: [(0, _env.withPulumiConfigPassphrase)()]
          })
        }
      });
    } catch (e) {
      stackExists = false;
    }
    if (!stackExists) {
      throw new Error(`Please specify an existing environment, for example "dev".`);
    }
  }
  const output = _simpleOutput.default;
  if (typeof output.initialize === "function") {
    output.initialize(inputs);
  }

  // Add deploy logs.
  if (inputs.deploy && projectApplication) {
    try {
      output.log({
        type: "deploy",
        message: _chalk.default.green("Watching cloud infrastructure resources...")
      });
      const buildFoldersGlob = [projectApplication.paths.workspace, "**/build/*.js"].join("/");
      const buildFolders = _fastGlob.default.sync(buildFoldersGlob, {
        onlyFiles: false
      });

      // The final array of values that will be sent to Pulumi CLI's "--path" argument.
      // NOTE: for Windows, there's a bug in Pulumi preventing us to use path filtering.
      let pathArg = undefined;
      if (PULUMI_WATCH_SUPPORTED) {
        pathArg = [...buildFolders];
        const pulumiFolder = _path.default.join(projectApplication.root, "pulumi");
        if (_fs.default.existsSync(pulumiFolder)) {
          pathArg.push(pulumiFolder);
        }
      }

      // Log used values if debugging has been enabled.
      if (inputs.debug) {
        const message = pathArg ? ["The following files and folders are being watched:", ...pathArg.map(p => "\n‣ " + p)].join("\n") : `Watching ${projectApplication.root}.`;
        output.log({
          type: "deploy",
          message
        });
      }
      const pulumi = await (0, _utils2.getPulumi)({
        projectApplication
      });
      const watchCloudInfrastructure = pulumi.run({
        command: "watch",
        args: {
          secretsProvider: PULUMI_SECRETS_PROVIDER,
          color: "always",
          path: pathArg,
          debug: !!inputs.debug
        },
        execa: {
          env: (0, _env.createEnvConfiguration)({
            configurations: [(0, _env.withRegion)(inputs), (0, _env.withEnv)(inputs), (0, _env.withEnvVariant)(inputs), (0, _env.withProjectName)(context)]
          })
        }
      });
      watchCloudInfrastructure.stdout.on("data", data => {
        const lines = data.toString().split("\n");
        for (let i = 0; i < lines.length; i++) {
          const line = lines[i];
          try {
            const [,, name, message] = line
            /**
             * TODO @adrian
             *
             * Check this out.
             */
            // added ! at the end of next line
            .match(/(.*)\[(.*)\] (.*)/).map(item => item.trim());
            if (name) {
              const coloredName = _chalk.default.hex((0, _utils2.getRandomColorForString)(name)).bold(name);
              output.log({
                type: "deploy",
                message: `${coloredName}: ${message}`
              });
            } else {
              output.log({
                type: "deploy",
                message
              });
            }
          } catch (e) {
            output.log({
              type: "deploy",
              message: line
            });
          }
        }
      });
      watchCloudInfrastructure.stderr.on("data", data => {
        output.log({
          type: "deploy",
          message: data.toString()
        });
      });
    } catch (e) {
      output.log({
        type: "deploy",
        message: _chalk.default.red(e.message)
      });
      if (inputs.debug) {
        output.log({
          type: "deploy",
          message: _chalk.default.red(e.stack)
        });
      }
    }
  }
  if (inputs.build) {
    try {
      await (0, _watchPackages.watchPackages)({
        inputs,
        context,
        output
      });
    } catch (e) {
      output.log({
        type: "build",
        message: _chalk.default.red(e.message)
      });
      if (inputs.debug) {
        output.log({
          type: "build",
          message: _chalk.default.red(e.stack)
        });
      }
    }
  }
};
exports.watchCommand = watchCommand;

//# sourceMappingURL=watch.js.map