"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createPulumiCommand = void 0;
var _path = _interopRequireDefault(require("path"));
var _utils = require("@webiny/cli/utils");
var _createProjectApplicationWorkspace = require("./createProjectApplicationWorkspace");
var _login = require("./login");
var _loadEnvVariables = require("./loadEnvVariables");
var _getPulumi = require("./getPulumi");
var _measureDuration = require("./measureDuration");
var _GracefulPulumiError = require("./GracefulPulumiError");
const createPulumiCommand = ({
  name,
  command,
  createProjectApplicationWorkspace: createProjectApplicationWorkspaceParam,
  telemetry
}) => {
  return async (params, context) => {
    // If folder not specified, that means we want to deploy the whole project (all project applications).
    // For that, we look if there are registered plugins that perform that.
    if (!params.folder) {
      const plugin = context.plugins.byName(`cli-command-deployment-${name}-all`);
      if (!plugin) {
        throw new _GracefulPulumiError.GracefulPulumiError(`Cannot continue - "cli-command-deployment-${name}-all" plugin not found.`);
      }
      return plugin[name](params, context);
    }

    // Before proceeding, let's detect if multiple project applications were passed.
    const folders = params.folder.split(",").map(current => current.trim());
    if (folders.length > 1) {
      for (let i = 0; i < folders.length; i++) {
        const folder = folders[i];
        await createPulumiCommand({
          name,
          command,
          createProjectApplicationWorkspace: _createProjectApplicationWorkspace.createProjectApplicationWorkspace
        })({
          ...params,
          folder
        }, context);
      }
      return;
    }

    // Detect if an app alias was provided.
    const project = (0, _utils.getProject)();
    if (project.config.appAliases) {
      const appAliases = project.config.appAliases;
      if (appAliases[params.folder]) {
        params.folder = appAliases[params.folder];
      }
    }
    const sendTelemetryEvents = telemetry === true && params.telemetry !== false;
    const getTelemetryEventName = stage => `cli-pulumi-command-${name}-${stage}`;
    const telemetryProperties = {
      env: params.env || "unknown",
      variant: params.variant || "unknown",
      commandParams: JSON.stringify(params)
    };
    let projectApplication = undefined;
    try {
      if (sendTelemetryEvents) {
        const eventName = getTelemetryEventName("start");
        await (0, _utils.sendEvent)(eventName, telemetryProperties);
      }
      if (!params.env) {
        throw new _GracefulPulumiError.GracefulPulumiError(`Please specify environment, for example "dev".`);
      }
      const getDuration = (0, _measureDuration.measureDuration)();
      const cwd = _path.default.join(process.cwd(), params.folder);

      // Get project application metadata.
      projectApplication = (0, _utils.getProjectApplication)({
        cwd
      });
      if (createProjectApplicationWorkspaceParam !== false) {
        await (0, _createProjectApplicationWorkspace.createProjectApplicationWorkspace)({
          projectApplication,
          inputs: params
        });
      }

      // Check if there are any plugins that need to be registered. This needs to happen
      // always, no matter the value of `createProjectApplicationWorkspaceParam` parameter.
      if (projectApplication.config.plugins) {
        context.plugins.register(projectApplication.config.plugins);
      }

      // Load env vars specified via .env files located in project application folder.
      await (0, _loadEnvVariables.loadEnvVariables)(params, context);
      await (0, _login.login)(projectApplication);
      const pulumi = await (0, _getPulumi.getPulumi)({
        projectApplication
      });
      const result = await command({
        inputs: params,
        context,
        projectApplication,
        pulumi,
        getDuration
      });
      if (sendTelemetryEvents) {
        const eventName = getTelemetryEventName("end");
        await (0, _utils.sendEvent)(eventName, telemetryProperties);
      }
      return result;
    } catch (e) {
      const gracefulError = _GracefulPulumiError.GracefulPulumiError.from(e, {
        projectApplication: {
          id: projectApplication?.id || "unknown"
        },
        commandParams: params
      });
      if (sendTelemetryEvents) {
        const eventName = gracefulError ? getTelemetryEventName("error-graceful") : getTelemetryEventName("error");
        await (0, _utils.sendEvent)(eventName, {
          ...telemetryProperties,
          errorMessage: e.message,
          errorStack: e.stack
        });
      }
      let message = "Command failed with an unexpected error. Please check the above logs.";
      if (!params.debug) {
        const debugFlag = context.error.hl(`--debug`);
        message += ` Alternatively, try running the same command with the ${debugFlag} flag to get more detailed information.`;
      }
      throw new Error(message, {
        cause: {
          error: e,
          gracefulError: gracefulError
        }
      });
    }
  };
};
exports.createPulumiCommand = createPulumiCommand;

//# sourceMappingURL=createPulumiCommand.js.map